/* This software was developed by Bruce Hendrickson and Robert Leland   *
 * at Sandia National Laboratories under US Department of Energy        *
 * contract DE-AC04-76DP00789 and is copyrighted by Sandia Corporation. */

#include	<stdio.h>
#include	"structs.h"
#include	"defs.h"

/* Make a bipartite graph from vertex separator and neighbors. */

void      make_bpgraph(graph, sets, bndy_list, sep_size, set_match,
			ppointers, pindices, ploc2glob, pnleft, pnright)
struct vtx_data **graph;	/* list of graph info for each vertex */
short    *sets;			/* local partitioning of vtxs */
int      *bndy_list;		/* list of vertices on boundary (0 ends) */
int       sep_size;		/* length of bndy_list */
int       set_match;		/* side to match against */
int     **ppointers;		/* start/stop of adjacency lists */
int     **pindices;		/* adjacency list for each vertex */
int     **ploc2glob;		/* maps bp number to full graph */
int      *pnleft;		/* number of nodes in left half */
int      *pnright;		/* number of nodes in right half */
{
    int      *loc2glob;		/* maps bp number to full graph */
    int      *pointers;		/* start/stop of adjacency lists */
    int      *indices;		/* adjacency list for each vertex */
    int      *vwgts;		/* saves vwgts so they can be overwritten */
    int       nleft, nright;	/* # vertices in halves of bipartite graph */
    int       index;		/* pointer into bp numbering */
    int       nedges;		/* number of edges in bipartite graph */
    int       vtx;		/* vertex in graph */
    int       neighbor;		/* neighbor of vertex */
    int       i, j, k;		/* loop counters */
    double   *smalloc();
    int       sfree();

    /* First count everything that needs to be counted. */
    nleft = sep_size;
    nright = 0;
    nedges = 0;
    for (i = 0; i < sep_size; i++) {
	vtx = bndy_list[i];
	for (j = 1; j < graph[vtx]->nedges; j++) {
	    neighbor = graph[vtx]->edges[j];
	    if (sets[neighbor] == set_match) {
		++nedges;
		if (graph[neighbor]->vwgt > 0) {
		    ++nright;
		    /* Flag him as seen already. */
		    graph[neighbor]->vwgt = -graph[neighbor]->vwgt;
		}
	    }
	}
    }

    pointers = (int *) smalloc((unsigned) (nleft + nright + 1) * sizeof(int));
    indices = (int *) smalloc((unsigned) (2 * nedges + 1) * sizeof(int));

    /* Now set up data structure to make construction easier */
    loc2glob = (int *) smalloc((unsigned) (nleft + nright + 1) * sizeof(int));
    vwgts = (int *) smalloc((unsigned) (nleft + nright + 1) * sizeof(int));
    for (i = 0; i < nleft; i++) {
	vtx = bndy_list[i];
	loc2glob[i] = vtx;
	/* Save glob2loc pointer in graph->vwgt */
	vwgts[i] = graph[vtx]->vwgt;
	graph[vtx]->vwgt = i;
    }
    k = nleft;
    for (i = 0; i < nleft; i++) {
	vtx = bndy_list[i];
	for (j = 1; j < graph[vtx]->nedges; j++) {
	    neighbor = graph[vtx]->edges[j];
	    if (sets[neighbor] == set_match) {
		if (graph[neighbor]->vwgt < 0) {
		    loc2glob[k] = neighbor;
		    /* Reflag him as seen already. */
		    /* Save glob2loc pointer in graph->vwgt */
		    vwgts[k] = -graph[neighbor]->vwgt;
		    graph[neighbor]->vwgt = k;
		    k++;
		}
	    }
	}
    }

    /* I can now construct graph directly */
    nedges = 0;
    pointers[0] = 0;
    for (i = 0; i < nleft; i++) {
	vtx = loc2glob[i];
	for (j = 1; j < graph[vtx]->nedges; j++) {
	    neighbor = graph[vtx]->edges[j];
	    if (sets[neighbor] == set_match) {
		index = graph[neighbor]->vwgt;
		indices[nedges++] = index;
	    }
	}
	pointers[i+1] = nedges;
    }

    for (i = nleft; i < nleft + nright; i++) {
	vtx = loc2glob[i];
	for (j = 1; j < graph[vtx]->nedges; j++) {
	    neighbor = graph[vtx]->edges[j];
	    if (sets[neighbor] == 2) {
		/* Find him in loc2glob list? */
		index = graph[neighbor]->vwgt;
		indices[nedges++] = index;
	    }
	}
	pointers[i+1] = nedges;
    }

    /* Now restore the vwgts. */
    for (i = 0; i < nleft + nright; i++) {
	graph[loc2glob[i]]->vwgt = vwgts[i];
    }

    sfree((char *) vwgts);

    *ploc2glob = loc2glob;
    *ppointers = pointers;
    *pindices = indices;
    *pnleft = nleft;
    *pnright = nright;
}
